library ptwebserver.config;
import 'dart:io';
import 'extHandler.dart';

import 'ext/def.dart';
import 'ext/php.dart';

/*
 * Main configuration to be sent to the server.
 */
class ptConfig {
  ptConfig() {
    hosts = new List<ptHost>();
    servername="ptWebServer";
    logPath="/var/log/ptws";
  }
  
  /*
   * Server's name
   */
  String servername;
  
  /*
   * Individual host configurations
   */
  List<ptHost> hosts;
  
  /*
   * Where to save log files
   */
  String logPath;
  
  Map toJson() {
    Map ret = new Map();
    ret['servername']=servername;
    ret['hosts']=hosts;
    return ret;
  }
}

typedef RequestCallback(HttpRequest req);


/**
 * A host configuration to be sent to the server handler
 */

class ptHost {
  ptHost({List<String> pub_index_files,List<extHandler> this.extensionHandlers}) {
    if(pub_index_files==null) {
      public_index_files = new List<String>();
    } else {
      public_index_files = pub_index_files;
    }
    
    hostnames = new List<String>();
    public_path = "";
    Request = null;
    allow_dir_view;
  }
  
  extHandler _default = new Default_Handler();
  extHandler get Default => _default;
  /*
   * Set the default handler for data
   */
  void set Default(extHandler val) { 
    _default=val; 
  }
  
  void SetDefaultHandlers() {
    if(extensionHandlers==null) {
      extensionHandlers= new List<extHandler>();
    }
    this.extensionHandlers.add(new PHP_Handler());
  }
  
  /*
   * Check to see if the hostname is in our list
   * Returns true if found, false otherwise
   */
  bool IsHostname(String name) {
    for(int i=0; i<hostnames.length; ++i) {
      if(hostnames[i].compareTo(name)==0) {
        return true;
      }
    }
    return false;
  }
  
  /*
   * Name of the host to be shown on the console
   */
  String name;
  
  /*
   * IP Address to bind to
   */
  String address = "127.0.0.1";
  
  /*
   * Hostnames to check against from the client
   */
  List<String> hostnames;
  
  /*
   * Port to bind to
   */
  int port = 80;
  
  /*
   * The path to look for when serving files
   */
  String public_path;
  
  /*
   * File names to look for when serving default pages such as index.html
   */
  List<String> public_index_files;
  
  /*
   * Allow the server to send out a directory view if no public_index_file is found.
   */
  bool allow_dir_view;
  
  Map toJson() {
    Map ret = new Map();
    ret['name']=name;
    ret['address']=address;
    ret['hostnames']=hostnames;
    ret['port']=port;
    ret['public_path']=public_path;
    ret['public_index_files'] = public_index_files;
    ret['allow_dir_view']=allow_dir_view;
    return ret;
  }
  
  /*
   * Override the request handler from the default
   * You are required to close the response.
   * host.Request = (HttpRequest request) {
   *  // Your code here
   * 
   *  request.response.close();
   * }
   */
  RequestCallback Request;
  
  List<extHandler> extensionHandlers;
}